/*
 * Copyright (C) 2000 by ETHZ/INF/CS
 * All rights reserved
 * 
 * @version $Id: Floor.java,v 1.1 2010/06/01 17:56:52 cpa Exp $
 * @author Roger Karrer
 */

import atomicsets.OrderedLock;
import java.lang.*;
import p.ArrayList;
import java.io.*;

// class to represent a floor in the control object
class Floor implements atomicsets.Atomic {

	/*atomicset(F)*/
	
    public final OrderedLock getLockForF() {
		return $lock_F;
	}
	public final OrderedLock getLock() {
		return this.getLockForF();
	}

	// Lists of people waiting to go up, and down
    // The Vectors will have instances of Integer objects.  The Integer will
    // store the floor that the person wants to go to
    /*atomic(F)*/ private ArrayList upPeople/*L=this.F*/;
    /*atomic(F)*/ private ArrayList downPeople/*L=this.F*/;

    // True if an elevator has claimed the the up or down call
    /*atomic(F)*/ private boolean upFlag;
    /*atomic(F)*/ private boolean downFlag;

    public boolean getUpFlag_internal(){ return upFlag; }
	public boolean getUpFlag() {
		synchronized (this.$lock_F) {
			return upFlag;
		}
	}
    public void setUpFlag_internal(boolean b){ upFlag = b; }
	public void setUpFlag(boolean b) {
		synchronized (this.$lock_F) {
			upFlag = b;
		}
	}
    public boolean getDownFlag_internal(){ return downFlag; }
	public boolean getDownFlag() {
		synchronized (this.$lock_F) {
			return downFlag;
		}
	}
    public void setDownFlag_internal(boolean b){ downFlag = b; }
	public void setDownFlag(boolean b) {
		synchronized (this.$lock_F) {
			downFlag = b;
		}
	}
    
    public ArrayList/*L=this.F*/ getUpPeople_internal(){ return upPeople; }
	public ArrayList getUpPeople() {
		synchronized (this.$lock_F) {
			return upPeople;
		}
	} 
    public void clearUpPeople_internal(){ upPeople = new ArrayList/*L=this.F*/(this.$lock_F); }
	public void clearUpPeople() {
		synchronized (this.$lock_F) {
			upPeople = new ArrayList/*L=this.F*/(this.$lock_F);
		}
	}
    public ArrayList/*L=this.F*/ getDownPeople_internal(){ return upPeople; }
	public ArrayList getDownPeople() {
		synchronized (this.$lock_F) {
			return upPeople;
		}
	} 
    public void clearDownPeople_internal(){ downPeople = new ArrayList/*L=this.F*/(this.$lock_F); }
	public void clearDownPeople() {
		synchronized (this.$lock_F) {
			downPeople = new ArrayList/*L=this.F*/(this.$lock_F);
		}
	}
    
    public Floor() {
		this(new OrderedLock());
	}
	public Floor(OrderedLock F) {
		$lock_F = F;
		upPeople = new ArrayList/*L=this.F*/(this.$lock_F);
		downPeople = new ArrayList/*L=this.F*/(this.$lock_F);
		upFlag = false;
		downFlag = false;
    }

	protected final OrderedLock $lock_F;
}
 
