/*
 * Copyright (C) 2000 by ETHZ/INF/CS
 * All rights reserved
 * 
 * @version $Id: Controls.java,v 1.1 2010/06/01 17:56:52 cpa Exp $
 * @author Roger Karrer
 */

import java.lang.*;
import p.ArrayList;
import java.io.*;

// class of the shared control object
class Controls {
    private Floor[] floors;
  
    public Controls(int numFloors) {
	floors = new Floor[numFloors+1];
	for(int i = 0; i <= numFloors; i++) floors[i] = new Floor();
    }

    // this is called to inform the control object of a down call on floor
    // onFloor
    public /*unitfor(floors[onFloor].F)*/ void pushDown(int onFloor, int toFloor) {
//	synchronized(floors[onFloor]) {
	    System.out.println("*** Someone on floor " + onFloor +
			       " wants to go to " + toFloor);
	    floors[onFloor].getDownPeople().add(new Integer(toFloor));
	    if(floors[onFloor].getDownPeople().size() == 1)
	       floors[onFloor].setDownFlag(false);
//	}
    }

    // this is called to inform the control object of an up call on floor
    // onFloor
    public /*unitfor(floors[onFloor].F)*/ void pushUp(int onFloor, int toFloor) {
//	synchronized(floors[onFloor]) {
	    System.out.println("*** Someone on floor " + onFloor +
			       " wants to go to " + toFloor);
	    floors[onFloor].getUpPeople().add(new Integer(toFloor));
	    if(floors[onFloor].getUpPeople().size() == 1)
	       floors[onFloor].setUpFlag(false);
//	}
    }

    // An elevator calls this if it wants to claim an up call
    // Sets the floor's upFlag to true if he has not already been set to true
    // Returns true if the elevator has successfully claimed the call, and
    // False if the call was already claimed (upFlag was already true)
    public /*unitfor(floors[floor].F)*/boolean claimUp(String lift, int floor) {
	if(checkUp(floor)) {
//	    synchronized(floors[floor]) {
		if(!floors[floor].getUpFlag()) {
		   floors[floor].setUpFlag(true);
		   return true;
//		}
	    }
	}
	return false;
    }

    // An elevator calls this if it wants to claim an down call
    // Sets the floor's downFlag to true if he has not already been set to true
    // Returns true if the elevator has successfully claimed the call, and
    // False if the call was already claimed (downFlag was already true)
    public /*unitfor(floors[floor].F)*/ boolean claimDown(String lift, int floor) {
	if(checkDown(floor)) {
//	    synchronized(floors[floor]) {
		if(!floors[floor].getDownFlag()) {
		    floors[floor].setDownFlag(true);
		    return true;
		}
//	    }
	}
	return false;
    }

    // An elevator calls this to see if an up call has occured on the given
    // floor.  If another elevator has already claimed the up call on the 
    // floor, checkUp() will return false.  This prevents an elevator from
    // wasting its time trying to claim a call that has already been claimed
    public /*unitfor(floors[floor].F)*/ boolean checkUp(int floor) {
//	synchronized(floors[floor]) { 
	    boolean ret = floors[floor].getUpPeople().size() != 0;
	    ret = ret && !floors[floor].getUpFlag();
	    return ret;
//	}
    }

    // An elevator calls this to see if a down call has occured on the given
    // floor.  If another elevator has already claimed the down call on the 
    // floor, checkUp() will return false.  This prevents an elevator from
    // wasting its time trying to claim a call that has already been claimed
    public /*unitfor(floors[floor].F)*/ boolean checkDown(int floor) {
//	synchronized(floors[floor]) {
	    boolean ret = floors[floor].getDownPeople().size() != 0;
	    ret = ret && !floors[floor].getDownFlag();
	    return ret;
//	}
    }

    // An elevator calls this to get the people waiting to go up.  The
    // returned Vector contains Integer objects that represent the floors
    // to which the people wish to travel.  The floors vector and upFlag
    // are reset.
    public /*unitfor(floors[floor].F)*/ ArrayList getUpPeople(int floor) {
//	synchronized(floors[floor]) {
		ArrayList temp = floors[floor].getUpPeople();
	    floors[floor].clearUpPeople();
	    floors[floor].setUpFlag(false);
	    return temp;
//	}
    }

    // An elevator calls this to get the people waiting to go down.  The
    // returned Vector contains Integer objects that represent the floors
    // to which the people wish to travel.  The floors vector and downFlag
    // are reset.
    public /*unitfor(floors[floor].F)*/ ArrayList getDownPeople(int floor) {
//	synchronized(floors[floor]) {
		ArrayList temp = floors[floor].getDownPeople();
	    floors[floor].clearDownPeople();
	    floors[floor].setDownFlag(false);
	    return temp;
//	}
    }
}


